SystemOrganization addCategory: #'GT-Playground'!

Object subclass: #GTPlaygroundContext
	instanceVariableNames: 'variableBindings notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Playground'!

!GTPlaygroundContext commentStamp: 'AndreiChis 2/27/2014 12:16' prior: 0!
I am context  that only holds a collection of binings.  I provide no bindings for temporary variables. 
Because I do not provide any bindings for temporary variables, any time an undeclared variable
is encountered it will be added to my list of binginds. For this to work properly I have to be set as
the requestor of the compiler.

|context|
context := GTPlaygroundContext new.
nil class compiler
		source: 'a:=1. b:=2. c:=a+b. c';
		context: nil; "context used to lookup variables; if nil no lookup is done"
		receiver: nil;
		requestor: context;
		failBlock:  [ ^ nil];
		evaluate.
context browseBindings.	"three bindings are present"!

!GTPlaygroundContext methodsFor: 'binding' stamp: 'AndreiChis 2/27/2014 16:58'!
bindingOf: aString 

	variableBindings ifNil: [self initializeBindings].
	(variableBindings includesKey: aString)
		ifFalse: [variableBindings at: aString put: nil].
	^ variableBindings associationAt: aString! !

!GTPlaygroundContext methodsFor: 'accessing' stamp: 'AndreiChis 2/27/2014 14:00'!
bingings

	^ variableBindings associations collect: [ :each| each key -> each value ]! !

!GTPlaygroundContext methodsFor: 'binding' stamp: 'AndreiChis 2/27/2014 11:38'!
browseBindings

	variableBindings inspect! !

!GTPlaygroundContext methodsFor: 'initialization' stamp: 'AndreiChis 2/27/2014 16:58'!
initialize

	super initialize.
	self initializeBindings.! !

!GTPlaygroundContext methodsFor: 'initialization' stamp: 'AndreiChis 2/27/2014 14:00'!
initializeBindings
	
	variableBindings := Dictionary new.! !

!GTPlaygroundContext methodsFor: 'as yet unclassified' stamp: 'AndreiChis 2/27/2014 18:17'!
notifier: anObject 
	notifier := anObject! !

!GTPlaygroundContext methodsFor: 'interactive error protocol' stamp: 'AndreiChis 2/27/2014 18:23'!
notify: aString at: anInteger in: aStream

	notifier notify: aString at: anInteger in: aStream! !

!GTPlaygroundContext methodsFor: 'accessing' stamp: 'AndreiChis 2/27/2014 12:07'!
setBindings: aDictionary

	variableBindings := aDictionary.
! !

Object subclass: #GTPlaygroundPage
	instanceVariableNames: 'title content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Playground'!

!GTPlaygroundPage commentStamp: 'TudorGirba 2/8/2014 22:59' prior: 0!
GTPlayground open!

!GTPlaygroundPage methodsFor: 'accessing' stamp: 'TudorGirba 2/3/2011 15:41'!
content
	^ content! !

!GTPlaygroundPage methodsFor: 'accessing' stamp: 'TudorGirba 2/3/2011 15:41'!
content: anObject
	content := anObject! !

!GTPlaygroundPage methodsFor: 'accessing' stamp: 'TudorGirba 2/8/2014 22:59'!
initialize
	content := ''.
	title := 'Playground'! !

!GTPlaygroundPage methodsFor: 'accessing' stamp: 'TudorGirba 2/3/2011 15:41'!
title
	^ title! !

!GTPlaygroundPage methodsFor: 'accessing' stamp: 'TudorGirba 2/3/2011 15:41'!
title: anObject
	title := anObject! !

GLMCompositePresentation subclass: #GTPlayground
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Playground'!

GTPlayground class
	instanceVariableNames: 'repository'!

!GTPlayground commentStamp: 'TudorGirba 2/10/2014 11:31' prior: 0!
self open.
self reset.

Playground is the correspondent of a classic Smalltalk Workspace. The problem with the name Workspace is that it implies that work should be carried out in this space, while this is not a best practice. Playground describes better the intention of providing a place in which we can quickly play with some code.

self registerToolsOn: Smalltalk tools
!

GTPlayground class
	instanceVariableNames: 'repository'!

!GTPlayground class methodsFor: 'instance creation' stamp: 'TudorGirba 2/9/2014 00:36'!
open
	"self open"
	| page |
	page := GTPlaygroundPage new.
	^ self new openOn: page! !

!GTPlayground class methodsFor: 'tools registry' stamp: 'TudorGirba 2/9/2014 01:10'!
registerToolsOn: registry
	"Add ourselves to registry. See [Smalltalk tools]"
	"self registerToolsOn: Smalltalk tools" 
	registry register: self as: #workspace
! !

!GTPlayground class methodsFor: 'accessing' stamp: 'TudorGirba 2/8/2014 22:54'!
repository
	^ repository ifNil: [ repository := OrderedCollection new ]! !

!GTPlayground class methodsFor: 'accessing' stamp: 'TudorGirba 2/9/2014 00:37'!
reset
	repository := nil! !

!GTPlayground class methodsFor: 'tools registry' stamp: 'TudorGirba 2/11/2014 22:23'!
taskbarIcon
	"Answer the icon for the receiver in a task bar."

	^ Smalltalk ui icons iconNamed: #workspaceIcon! !

!GTPlayground methodsFor: 'building' stamp: 'TudorGirba 2/10/2014 06:38'!
actionsForPreviousContents
	^ self class repository reverse
		collect: [ :each | 
			GLMGenericAction new
				title: (each content asString contractTo: 50);
				action: [ :text | 
					self pushToRepositoryFrom: text.
					text entity: each.
					self class repository remove: each.
					text update ];
				yourself ]! !

!GTPlayground methodsFor: 'building' stamp: 'AndreiChis 2/27/2014 17:58'!
codeIn: a
	^ a smalltalkCode
		title: [ :page | page title ];
		format: [ :page | page content ];
		variableBindings: [ :page| context bingings ];
		selectionPopulate: #selection
			on: $o
			entitled: 'Open'
			with: [ :text :page |
				page content: text text.
				self evaluateSelectionFrom: text ];
		dynamicActions: [ self actionsForPreviousContents ]! !

!GTPlayground methodsFor: 'building' stamp: 'TudorGirba 2/9/2014 01:12'!
compose
	self
		title: 'Playground';
		custom:
			(GTInspector new first
				noActions;
				showFirst: [ :a | self codeIn: a ];
				yourself);
		on: GLMBrowserClosing do: [ 
			"This is a hack for the moment.
			We need a better mechanism from Glamour to help 
			us keep the model in sync"
			self entity
					content: self first panes first presentations first text.
			self entity content asString trimBoth isEmpty
				ifFalse: [ self class repository add: self entity ] ]! !

!GTPlayground methodsFor: 'building' stamp: 'AndreiChis 2/27/2014 18:37'!
evaluateSelectionFrom: aTextPresentation
	| result |
	
	aTextPresentation selectLine.
	context notifier: aTextPresentation. "In case of a compilation error this object get's notified."
	result := self evaluateText: aTextPresentation selectedText notifying: context.
	context notifier: nil. 
	aTextPresentation updateVariableBindings.
	^ result! !

!GTPlayground methodsFor: 'do-its' stamp: 'AndreiChis 2/27/2014 18:01'!
evaluateText: aText notifying: aRequestor
	|result|
	
	"Must evaluate with context and receiver nil so that the missing bingings get created"
	result := nil class compiler
		source: aText;
		context: nil;
		receiver: nil;
		requestor: aRequestor;
		failBlock:  [ ^ nil];
		evaluate.	
	^ result
! !

!GTPlayground methodsFor: 'initializing' stamp: 'AndreiChis 2/27/2014 18:06'!
initialize

	super initialize.
	context := GTPlaygroundContext new! !

!GTPlayground methodsFor: 'building' stamp: 'TudorGirba 2/9/2014 11:42'!
pushToRepositoryFrom: aTextPresentation
	| page |
	page := GTPlaygroundPage new.
	page content: aTextPresentation text.
	page content asString trimBoth isEmpty
		ifFalse: [ self class repository add: page ] ! !
